/*
 * Copyright 1997 Vectaport Inc.
 * Copyright 1994,1995 Vectaport Inc., Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <Map/mapcov.h>
#include <Map/mapdb.h>
#include <Map/mapfclass.h>
#include <Map/mapfeature.h>
#include <Map/maplib.h>
#include <Map/mapprim.h>
#include <Map/mapproject.h>

#include <MapUnidraw/mapclasses.h>
#include <MapUnidraw/mapcmds.h>
#include <MapUnidraw/mapcomps.h>
#include <MapUnidraw/mapeditor.h>
#include <MapUnidraw/mapscripts.h>
#include <MapUnidraw/mapviews.h>

#include <OverlayUnidraw/paramlist.h>

#include <Unidraw/unidraw.h>
#include <Unidraw/ulist.h>
#include <Unidraw/viewer.h>

#include <Unidraw/Commands/command.h>

#include <Unidraw/Graphic/damage.h>
#include <Unidraw/Graphic/graphic.h>

#include <Unidraw/iterator.h>

#include <Attribute/aliterator.h>
#include <Attribute/attribute.h>
#include <Attribute/attrlist.h>
#include <Attribute/attrvalue.h>

#include <TopoFace/fgeomobjs.h>
#include <TopoFace/topoedge.h>
#include <TopoFace/toponode.h>

#include <OS/math.h>

#include <stdio.h>
#include <stream.h>
#include <string.h>

/****************************************************************************/

ParamList* MapFeatureComp::_map_feat_params = nil;

MapFeatureComp::MapFeatureComp(MapFeature* mf, OverlayComp* parent) : 
OverlayComp(nil, parent) {
    _feature = mf;
    if (_feature)
        SetAttributeList(_feature->attrlist());
    else 
        _attrlist = nil;
    _pointcomp = nil;
}

MapFeatureComp::MapFeatureComp (istream& in, MapFeature::MapFeatureType type, OverlayComp* parent) : OverlayComp(nil, parent) {
    _feature = new MapFeature(-1, type);
    _valid = GetParamList()->read_args(in, this);
    if (_attrlist && _attrlist->GetAttr("id")) {
      Attribute* idattr = _attrlist->GetAttr("id");
      AttributeValue* idav = idattr->Value();
      if (idav && idav->type() == AttributeValue::IntType)
	_feature->id(idav->int_val());
    }
    _pointcomp = nil;
}

MapFeatureComp::~MapFeatureComp() {
    _gr = nil;
}

ParamList* MapFeatureComp::GetParamList() {
    if (!_map_feat_params) 
	GrowParamList(_map_feat_params = new ParamList());
    return _map_feat_params;
}

void MapFeatureComp::GrowParamList(ParamList* pl) {
    pl->add_param("feature", ParamStruct::required, &MapFeatureScript::ReadFeature, this, &_feature);
    pl->add_param("gs", ParamStruct::keyword,         &OverlayScript::ReadGS,
	this, this, &_gr);
    pl->add_param("other", ParamStruct::other,  &OverlayScript::ReadOther,
	this, this);
}

void MapFeatureComp::feature(MapFeature* mf) {
    _feature = mf;
    _attrlist = _feature->attrlist();
}

MapFeature* MapFeatureComp::feature() {
    return _feature;
}

void MapFeatureComp::graphic(Graphic* g) {
    SetGraphic(g);
}

Component* MapFeatureComp::Copy() {
    MapFeatureComp* mfc = new MapFeatureComp(_feature);
    mfc->graphic(_gr);
    return mfc;
}

ClassId MapFeatureComp::GetClassId () { return MAPFEATURE_COMP; }

boolean MapFeatureComp::IsA (ClassId id) {
    return MAPFEATURE_COMP == id || OverlayComp::IsA(id);
}

GraphicComp* MapFeatureComp::GetGraphicComp (Graphic* g) {
    return (GraphicComp*) ((GraphicView*)g->GetTag())->GetGraphicComp();
}

void MapFeatureComp::update(Observable* obs) {
  cerr << "update received from observable on map feature comp\n";
}

/*****************************************************************************/

ParamList* MapFClassComp::_map_fcl_params = nil;

MapFClassComp::MapFClassComp(MapFeatureClass* mfc) : OverlaysComp() {
    _mapfc = mfc;
    _pathname = nil;
}

MapFClassComp::MapFClassComp (istream& in, OverlayComp* parent) : OverlaysComp(parent) {
    ResetIndexedGS();
    _mapfc = nil;
    _pathname = nil;
    _valid = GetParamList()->read_args(in, this);
    Iterator i;
    int nfeats = 0;
    for (First(i); !Done(i); Next(i)) nfeats++;
    MapFeature** feats = new MapFeature*[nfeats];
    int j = 0;
    for (First(i); !Done(i); Next(i)) {
        feats[j++] = ((MapFeatureComp*)GetComp(i))->feature();
        feats[j-1]->graphic((void*)GetComp(i));
    }
    _mapfc = new MapFeatureClass(nfeats, feats);
    AttributeList *al = attrlist();
    if (al) {
      AttributeValue* x1val = al->find("sw_e");
      AttributeValue* y1val = al->find("sw_n");
      AttributeValue* x2val = al->find("ne_e");
      AttributeValue* y2val = al->find("ne_n");
      AttributeValue* zoneval = al->find("zone");
      if (x1val && y1val && x2val && y2val && zoneval) {
	_mapfc->set_extent(x1val->double_val(), y1val->double_val(), 
			   x2val->double_val(), y2val->double_val(), 
			   zoneval->int_val());
#if 0
	if (parent && parent->IsA(MAP_IDRAW_COMP)) {
	  // copy attrs
	  AttributeList* pal = parent->GetAttributeList();
	  if (!pal->GetAttr("sw_e") && !pal->GetAttr("sw_n") &&
	      !pal->GetAttr("ne_e") && !pal->GetAttr("ne_n")) {
	    pal->Append(new Attribute(*al->GetAttr("sw_e")));
	    pal->Append(new Attribute(*al->GetAttr("sw_n")));
	    pal->Append(new Attribute(*al->GetAttr("ne_e")));
	    pal->Append(new Attribute(*al->GetAttr("ne_n")));
	  }
	}
#endif
      }
    }
    // let's get junction and line primitives connected
    for (int k = 0; k < nfeats; k++)
      feats[k]->feature_class(_mapfc);
    _mapfc->make_junction_idx();
    _mapfc->make_line_idx();
    MapFeature* junction = _mapfc->first_junction();
    while (junction) {
      Attribute* conns = ((MapFeatureComp*)junction->graphic())->GetAttributeList()->GetAttr("connections");
      if (conns) {
	MapFeature* line;
	AttributeValueList* list;
	ALIterator ali;
	switch(conns->Value()->type()) {
	case AttributeValue::IntType:
	  line = _mapfc->line_by_id(abs(conns->Value()->int_val()));
	  if (line) {
	    junction->node_primitive()->append(line->edge_primitive());
	    const float* nx = junction->node_primitive()->xpoints();
	    const float* ny = junction->node_primitive()->ypoints();
	    int lnp = line->edge_primitive()->npts();
	    const float* lx = line->edge_primitive()->xpoints();
	    const float* ly = line->edge_primitive()->ypoints();
	    if (nx[0] == lx[0] && ny[0] == ly[0])
	      line->edge_primitive()->topo()->attach_start_node(junction->node_primitive()->topo());
	    if (nx[0] == lx[lnp-1] && ny[0] == ly[lnp-1])
	      line->edge_primitive()->topo()->attach_end_node(junction->node_primitive()->topo());
	  }
	  break;
	case AttributeValue::ArrayType:
	  list = conns->Value()->array_val();
	  for (list->First(ali); !list->Done(ali); list->Next(ali)) {
	    AttributeValue* val = list->GetAttrVal(ali);
	    line = _mapfc->line_by_id(abs(val->int_val()));
	    if (line) {
	      junction->node_primitive()->append(line->edge_primitive());
	    const float* nx = junction->node_primitive()->xpoints();
	    const float* ny = junction->node_primitive()->ypoints();
	    int lnp = line->edge_primitive()->npts();
	    const float* lx = line->edge_primitive()->xpoints();
	    const float* ly = line->edge_primitive()->ypoints();
	    if (nx[0] == lx[0] && ny[0] == ly[0])
	      line->edge_primitive()->topo()->attach_start_node(junction->node_primitive()->topo());
	    if (nx[0] == lx[lnp-1] && ny[0] == ly[lnp-1])
	      line->edge_primitive()->topo()->attach_end_node(junction->node_primitive()->topo());
	    }
	  }
	  break;
	default:
	  break;
	}
      }
      junction = _mapfc->next_junction();
    }
}

ParamList* MapFClassComp::GetParamList() {
    if (!_map_fcl_params) 
	GrowParamList(_map_fcl_params = new ParamList());
    return _map_fcl_params;
}

void MapFClassComp::GrowParamList(ParamList* pl) {
    pl->add_param("kids", ParamStruct::optional, &OverlaysScript::ReadChildren, this, this);
    pl->add_param("path", ParamStruct::keyword, &MapFClassScript::ReadPath, this, this);
    pl->add_param("file", ParamStruct::keyword, &MapFClassScript::ReadFile, this, this);
    OverlayComp::GrowParamList(pl);
}

MapFClassComp::~MapFClassComp() {
    Iterator i;

    First(i);
    while (!Done(i)) {
        GetComp(i)->SetGraphic(nil);
	Next(i);
    }
}

void MapFClassComp::Interpret(Command* cmd) {
    if (cmd->IsA(MAPPROJECTION_CMD)) {
	Iterator i;
	for (First(i); !Done(i); Next(i)) {
            GetComp(i)->Notify();
        }
    }
    else
	OverlaysComp::Interpret(cmd);
}

void MapFClassComp::Uninterpret(Command* cmd) {
}

Component* MapFClassComp::Copy() { return new MapFClassComp(_mapfc); }
ClassId MapFClassComp::GetClassId() { return MAPFCLASS_COMP; }
boolean MapFClassComp::IsA(ClassId id) {
    return id == MAPFCLASS_COMP || OverlaysComp::IsA(id);
}

void MapFClassComp::SetPathName(const char* p) {
    if (_mapfc) _mapfc->set_path(p);
    delete _pathname;
    _pathname = strdup(p);
}

const char* MapFClassComp::GetPathName() {
    if (_mapfc && _mapfc->path()) 
        return _mapfc->path();
    else
        return _pathname;
}

unsigned char* MapFClassComp::bitmap
(float scale, int& ncols, int& nrows, 
 float& xorigin, float& yorigin, int& utmzone) 
{
  MapFeatureClass* fclass = GetFeatureClass();
  if (!fclass) return nil;
  double x1, y1, x2, y2;
  fclass->extent(x1, y1, x2, y2, utmzone);
  if (utmzone==0) {
    utmzone = MapProjection::utmzone(x1);
    char lon0[80];
    sprintf(lon0, "lon_0=%f", x1);
    char* parms[4];
    parms[0] = "proj=utm";
    parms[1] = "ellps=WGS84";
    parms[2] = lon0;
    parms[3] = "no_defs";
    PJ* proj = pj_init(4, parms);
    UV data;
    data.u = x1 * DEG_TO_RAD;
    data.v = y1 * DEG_TO_RAD;
    data = pj_fwd(data, proj);
    x1 = data.u;
    y1 = data.v;
    data.u = x2 * DEG_TO_RAD;
    data.v = y2 * DEG_TO_RAD;
    data = pj_fwd(data, proj);
    x2 = data.u;
    y2 = data.v;
  }
  
  int xmin, ymin, xmax, ymax;
  xmin = Math::round(Math::min(x1, x2)/scale);
  ymin = Math::round(Math::min(y1, y2)/scale);
  xorigin = xmin;
  yorigin = ymin;
  xmax = Math::round(Math::max(x1, x2)/scale);
  ymax = Math::round(Math::max(y1, y2)/scale);
  ncols = xmax-xmin+1;
  nrows = ymax-ymin+1;
  unsigned char* bm = new unsigned char[ncols*nrows];
  for (int i=0; i<ncols*nrows; i++) 
    bm[i] = 0;

  Iterator i;
  for (First(i); !Done(i); Next(i)) {
    MapFeatureComp* fcomp = (MapFeatureComp*)GetComp(i);
    MapFeature* feat = fcomp->feature();
    switch (feat->type()) {
    case MapFeature::PointType: {
      MapPoint* point = feat->point_primitive();
      int xloc = Math::round(point->xpoints()[0]) - xmin;
      int yloc = Math::round(point->ypoints()[0]) - ymin;
      bm[xloc+yloc*nrows] = 1;
    }
    break;
    case MapFeature::LineType: {
      MapEdge* edge = feat->edge_primitive();
      int npts = edge->npts();
      float* fx = (float*)edge->xpoints();
      float* fy = (float*)edge->ypoints();
      float newx[npts];
      float newy[npts];
      for (int ii=0; ii<npts; ii++) {
	newx[ii] = fx[ii]/scale;
	newy[ii] = fy[ii]/scale;
      }
      FMultiLineObj fl(newx, newy, npts);
      int *xpts, *ypts;
      int len = fl.Bresenham(xpts, ypts);
      for (int ii=0; ii<len; ii++) {
	int xloc = xpts[ii] - xmin;
	int yloc = ypts[ii] - ymin;
	if (xloc>0 && yloc>0 && xloc<ncols && yloc<nrows)
	  bm[xloc + (nrows-yloc-1)*ncols] = 1;
      }
    }
    break;
    case MapFeature::AreaType: {
      MapFace* face = feat->face_primitive();
      int npts = face->npts();
      float* fx = (float*)face->xpoints();
      float* fy = (float*)face->ypoints();
      float newx[npts];
      float newy[npts];
      for (int ii=0; ii<npts; ii++) {
	newx[ii] = fx[ii]/scale;
	newy[ii] = fy[ii]/scale;
      }
      FFillPolygonObj fp(newx, newy, npts);
      int *ylocs, *xbegs, *xends;
      boolean *xings;
      int len = fp.SortedBorders(ylocs, xbegs, xends, xings);
      int curr = 0;
      boolean inside;
      while (curr<len) {
	int yloc = ylocs[curr]-ymin;
	if (yloc<0 || yloc>=nrows) continue;

	/* always paint the border */
	int xstart = Math::max(xbegs[curr]-xmin, 0);
	int xstop = Math::min(xends[curr]-xmin,ncols-1);
	for (int xloc=xstart; xloc<=xstop; xloc++) 
	  bm[xloc + (nrows-yloc-1)*ncols] = 1;

	/* determine whether inside or out after border */
	boolean newy = curr ? ylocs[curr-1]-ymin != yloc : 1;
	if (newy) inside = false;
	if (xings[curr]) inside = !inside;

	/* paint every inside pixel between borders */
	if (inside) {
	  xstart = Math::min(xstop+1, xbegs[curr+1]-xmin-1);
	  xstop = xbegs[curr+1]-xmin-1;
	  for (int xloc=xstart; xloc<=xstop; xloc++) 
	    bm[xloc + (nrows-yloc-1)*ncols] = 1;
	}

	curr++;
      }
    }
    break;
    }
  }

  return bm;
  
}

/*****************************************************************************/

ParamList* MapCovComp::_map_cov_params = nil;

MapCovComp::MapCovComp(MapCoverage* mc) : OverlaysComp() {
    _mapcov = mc;
}

MapCovComp::MapCovComp (istream& in) {
    _valid = GetParamList()->read_args(in, this);
}

ParamList* MapCovComp::GetParamList() {
    if (!_map_cov_params) 
	GrowParamList(_map_cov_params = new ParamList());
    return _map_cov_params;
}

void MapCovComp::GrowParamList(ParamList* pl) {
    OverlaysComp::GrowParamList(pl);
}

MapCovComp::~MapCovComp()
{
}

MapFClassComp* MapCovComp::FindFClassComp(MapFeatureClass* mapfc) {
    Iterator i;
    for (First(i); !Done(i); Next(i)) {
	MapFClassComp* mfc = (MapFClassComp*)GetComp(i);
	if (mfc->GetFeatureClass() == mapfc)
	    return mfc;
    }
    return nil;
}

void MapCovComp::Interpret(Command* cmd) {
    if (cmd->IsA(MAPSHOWALL_CMD) || cmd->IsA(MAPPROJECTION_CMD)) {
	Iterator i;
	for (First(i); !Done(i); Next(i)) {
            GetComp(i)->Interpret(cmd);
        }
    }
    else if (cmd->IsA(MAPSHOW_CMD)) {
	MapFeatureClass* mapfc = (MapFeatureClass*)_mapcov->feature_class(((MapShowCmd*)cmd)->GetFClassName());
	if (mapfc) {
	    MapFClassComp* mfc = FindFClassComp(mapfc);
	    if (!mfc) {
		mfc = CreateFClassComp(mapfc);
		Append(mfc);
		Notify();
		mfc->Interpret(cmd);
	    }
	    else
		((MapEditor*)cmd->GetEditor())->GetMapDbView()->Interpret(cmd);
	}
    }
    else if (cmd->IsA(MAPHIDE_CMD)) {
	MapFeatureClass* mapfc = (MapFeatureClass*)_mapcov->feature_class(((MapHideCmd*)cmd)->GetFClassName());
	if (mapfc) {
	    MapFClassComp* mfc = FindFClassComp(mapfc);
	    if (mfc) {
		((MapEditor*)cmd->GetEditor())->GetMapDbView()->Interpret(cmd);
		mfc->Interpret(cmd);
	    }
	}
    }
    else
	OverlaysComp::Interpret(cmd);
}

MapFClassComp* MapCovComp::CreateFClassComp(MapFeatureClass* mfc) {
    return new MapFClassComp(mfc);
}

void MapCovComp::Uninterpret(Command* cmd) {
}

Component* MapCovComp::Copy() { return new MapCovComp(_mapcov); }
ClassId MapCovComp::GetClassId() { return MAPCOV_COMP; }
boolean MapCovComp::IsA(ClassId id) {
    return id == MAPCOV_COMP || OverlaysComp::IsA(id);
}

/*****************************************************************************/

ParamList* MapLibComp::_map_lib_params = nil;

MapLibComp::MapLibComp(MapLibrary* ml) : OverlaysComp() {
    _maplib = ml;
}

MapLibComp::MapLibComp (istream& in) {
    _valid = GetParamList()->read_args(in, this);
}

ParamList* MapLibComp::GetParamList() {
    if (!_map_lib_params) 
	GrowParamList(_map_lib_params = new ParamList());
    return _map_lib_params;
}

void MapLibComp::GrowParamList(ParamList* pl) {
    OverlaysComp::GrowParamList(pl);
}

MapLibComp::~MapLibComp() {
}

MapCovComp* MapLibComp::FindCovComp(MapCoverage* mapcov) {
    Iterator i;
    for (First(i); !Done(i); Next(i)) {
	MapCovComp* mcc = (MapCovComp*)GetComp(i);
	if (mcc->GetCoverage() == mapcov)
	    return mcc;
    }
    return nil;
}

void MapLibComp::Interpret(Command* cmd) {
    if (cmd->IsA(MAPSHOWALL_CMD) || cmd->IsA(MAPPROJECTION_CMD)) {
	Iterator i;
	for (First(i); !Done(i); Next(i)) {
            GetComp(i)->Interpret(cmd);
        }
    } else if (cmd->IsA(MAPSHOW_CMD)) {
	MapCoverage* mapcov = (MapCoverage*)_maplib->coverage(((MapShowCmd*)cmd)->GetCovName());
	if (mapcov) {
	    MapCovComp* mcc = FindCovComp(mapcov);
	    if (!mcc) {
		mcc = CreateCovComp(mapcov);
		Append(mcc);
		Notify();
	    }
	    mcc->Interpret(cmd);
	}
    }
    else
    if (cmd->IsA(MAPHIDE_CMD)) {
	MapCoverage* mapcov = (MapCoverage*)_maplib->coverage(((MapHideCmd*)cmd)->GetCovName());
	if (mapcov) {
	    MapCovComp* mcc = FindCovComp(mapcov);
	    if (mcc)
		mcc->Interpret(cmd);
	}
    }
    else
	OverlaysComp::Interpret(cmd);
}

MapCovComp* MapLibComp::CreateCovComp(MapCoverage* cov) {
    return new MapCovComp(cov);
}

void MapLibComp::Uninterpret(Command* cmd) {
}

Component* MapLibComp::Copy() { return new MapLibComp(_maplib); }
ClassId MapLibComp::GetClassId() { return MAPLIB_COMP; }
boolean MapLibComp::IsA(ClassId id) {
    return id == MAPLIB_COMP || OverlaysComp::IsA(id);
}

/*****************************************************************************/

ParamList* MapDbComp::_map_db_params = nil;

MapDbComp::MapDbComp(MapDatabase* mdb) : OverlaysComp()
{
    _mapdb = mdb;
}

MapDbComp::MapDbComp (istream& in) {
    _valid = GetParamList()->read_args(in, this);
}

ParamList* MapDbComp::GetParamList() {
    if (!_map_db_params)
	GrowParamList(_map_db_params = new ParamList());
    return _map_db_params;
}

void MapDbComp::GrowParamList(ParamList* pl) {
    OverlaysComp::GrowParamList(pl);
}

MapDbComp::~MapDbComp()
{
}

MapLibComp* MapDbComp::FindLibComp(MapLibrary* maplib) {
    Iterator i;
    for (First(i); !Done(i); Next(i)) {
	MapLibComp* mlc = (MapLibComp*)GetComp(i);
	if (mlc->GetLibrary() == maplib)
	    return mlc;
    }
    return nil;
}

void MapDbComp::Interpret(Command* cmd) {
    if (cmd->IsA(MAPSHOWALL_CMD) || cmd->IsA(MAPPROJECTION_CMD)) {
	Iterator i;
	for (First(i); !Done(i); Next(i)) {
            GetComp(i)->Interpret(cmd);
        }
    }
    else if (cmd->IsA(MAPSHOW_CMD)) {
      if (_mapdb) {
	MapLibrary* maplib = (MapLibrary*)_mapdb->library(((MapShowCmd*)cmd)->GetLibName());
	if (maplib) {
	    MapLibComp* mlc = FindLibComp(maplib);
	    if (!mlc) {
		mlc = CreateLibComp(maplib);
		Append(mlc);
		Notify();
	    }
	    mlc->Interpret(cmd);
	}
      }
    }
    else if (cmd->IsA(MAPHIDE_CMD)) {
	MapLibrary* maplib = (MapLibrary*)_mapdb->library(((MapHideCmd*)cmd)->GetLibName());
	if (maplib) {
	    MapLibComp* mlc = FindLibComp(maplib);
	    if (mlc)
		mlc->Interpret(cmd);
	}
    }
    else
	OverlaysComp::Interpret(cmd);
}

MapLibComp* MapDbComp::CreateLibComp(MapLibrary* ml) {
    return new MapLibComp(ml);
}

void MapDbComp::Uninterpret(Command* cmd) {
}

Component* MapDbComp::Copy() { return new MapDbComp(_mapdb); }
ClassId MapDbComp::GetClassId() { return MAPDB_COMP; }
boolean MapDbComp::IsA(ClassId id) {
    return id == MAPDB_COMP || OverlaysComp::IsA(id);
}

/****************************************************************************/

int MapIdrawComp::_projection_symid = -1;
ParamList* MapIdrawComp::_map_idraw_params = nil;

MapIdrawComp::MapIdrawComp (const char* pathname, OverlayComp* parent) : FrameIdrawComp(true, pathname, parent) 
{ 
  if (_projection_symid==-1)
    _projection_symid = symbol_add("projection");

}

MapIdrawComp::MapIdrawComp (boolean add_bg, const char* pathname, OverlayComp* parent) : FrameIdrawComp(add_bg, pathname, parent) 
{ 
  if (_projection_symid==-1)
    _projection_symid = symbol_add("projection");

}

ClassId MapIdrawComp::GetClassId () { return MAP_IDRAW_COMP; }

boolean MapIdrawComp::IsA (ClassId id) {
    return MAP_IDRAW_COMP == id || FrameIdrawComp::IsA(id);
}

MapIdrawComp::MapIdrawComp (istream& in, const char* pathname, OverlayComp* parent) : FrameIdrawComp(true, pathname, parent) {
    _valid = GetParamList()->read_args(in, this);
#if 0
    // if no sw/ne attributes then pick up from first child that has them
    AttributeList* al = GetAttributeList();
    if (!al->GetAttr("sw_e") || !al->GetAttr("sw_n") ||
	!al->GetAttr("ne_e") || !al->GetAttr("ne_n")) {
      Iterator i;
      for (First(i); !Done(i); Next(i)) {
	OverlayComp* comp = (OverlayComp*)GetComp(i);
	AttributeList* cal = comp->GetAttributeList();
	if (cal->GetAttr("sw_e") && cal->GetAttr("sw_n") &&
	    cal->GetAttr("ne_e") && cal->GetAttr("ne_n")) {
	  al->Append(new Attribute(*cal->GetAttr("sw_e")));
	  al->Append(new Attribute(*cal->GetAttr("sw_n")));
	  al->Append(new Attribute(*cal->GetAttr("ne_e")));
	  al->Append(new Attribute(*cal->GetAttr("ne_n")));
	  if (cal->GetAttr("proj"))
	    al->Append(new Attribute(*cal->GetAttr("proj")));
	  if (cal->GetAttr("ellps"))
	    al->Append(new Attribute(*cal->GetAttr("ellps")));
	  if (cal->GetAttr("zone"))
	    al->Append(new Attribute(*cal->GetAttr("zone")));
	  break;
	}
      }
    }
#endif
}

ParamList* MapIdrawComp::GetParamList() {
    if (!_map_idraw_params) 
	GrowParamList(_map_idraw_params = new ParamList());
    return _map_idraw_params;
}

void MapIdrawComp::GrowParamList(ParamList* pl) {
    FrameIdrawComp::GrowParamList(pl); 
}

Component* MapIdrawComp::Copy () {
    return new MapIdrawComp((const char*)nil, nil);
}

const char* MapIdrawComp::GetProjectionName() 
{ 
  AttributeList* al = attrlist();
  if (al) {
    AttributeValue* av = al->find(_projection_symid);
    return av->symbol_ptr();
  } else
    return nil;
}

void MapIdrawComp::SetProjectionName(char* p) { 
  AttributeList* al = attrlist();
  if (!al) {
    al = new AttributeList();
    SetAttributeList(al);
  }
  Attribute* attr = 
    new Attribute(_projection_symid, 
		  new AttributeValue(p));
  al->add_attr(attr);
}
