/*
 * Copyright (c) 1998 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <ProjServ/projfunc.h>

#include <VpfUnidraw/vpfclasses.h>
#include <VpfUnidraw/vpfcomps.h>
#include <VpfUnidraw/vpfcreator.h>
#include <VpfUnidraw/vpfscripts.h>

#include <Vpf/vpfdb.h>
#include <Vpf/vpflib.h>
#include <Vpf/vpfcov.h>
#include <Vpf/vpffclass.h>

#include <MapServ/mapfunc.h>

#include <MapUnidraw/mapcmds.h>

#include <Map/mapfeature.h>
#include <Map/mapprim.h>

#include <ComTerp/comterpserv.h>
#include <ComTerp/comvalue.h>

extern "C" {
#include <projects.h>
}

#include <strstream.h>
#include <stdio.h>

LoadVpfFCFunc::LoadVpfFCFunc(ComTerp* comterp) : ComFunc(comterp) {
  _tileid_symid = symbol_add("tileid");
}

void LoadVpfFCFunc::execute() {
    ComValue dbpath(stack_arg(0));
    ComValue libname(stack_arg(1));
    ComValue covname(stack_arg(2));
    ComValue fcname(stack_arg(3));
    ComValue tileid(stack_key(_tileid_symid));
    reset_stack();
    ComValue result;

    Creator::instance(new VpfCreator());
    const char* dbdir = dbpath.string_ptr();
    VpfDatabase* db = new VpfDatabase(dbdir);
    VpfLibrary* lib = db->library(libname.string_ptr());
    VpfDbComp* dbcomp = new VpfDbComp(db);
    MapShowCmd* showcmd = new MapShowCmd();
    showcmd->SetUseEditor(false);
    showcmd->SetLibName(libname.string_ptr());
    showcmd->SetCovName(covname.string_ptr());
    showcmd->SetFClassName(fcname.string_ptr());
    VpfExtent* ext = lib->extent();
    char lon0[80];
    sprintf(lon0, "lon_0=%f", ext->x1);  // let proj pick the utm zone
    char* parms[4];
    parms[0] = "proj=utm";
    parms[1] = "ellps=WGS84";
    parms[2] = lon0;
    parms[3] = "no_defs";
    PJ* proj = pj_init(4, parms);
    //showcmd->SetProjection(proj);
    if (tileid.type() != ComValue::UnknownType) {
      int tid = tileid.int_val();
      showcmd->SetTileId(tid);
    }
    dbcomp->Interpret(showcmd);

    VpfLibComp* libcomp = (VpfLibComp*)dbcomp->FindLibComp(lib);
    VpfCoverage* cov = lib->coverage(covname.string_ptr());
    VpfCovComp* covcomp = (VpfCovComp*)libcomp->FindCovComp(cov);
    VpfFeatureClass* fclass = cov->feature_class(fcname.string_ptr());
    VpfFClassComp* fccomp = (VpfFClassComp*)covcomp->FindFClassComp(fclass);
    ComponentView* view = new ComponentView(fccomp);
    result.type(AttributeValue::ObjectType);
    result.obj_ref() = view;
    result.obj_type_ref() = FEATURE_CLASS;
    push_stack(result);
}

GraticuleFCFunc::GraticuleFCFunc(ComTerp* comterp) : ComFunc(comterp) {
  _startlon_symid = symbol_add("startlon");
  _endlon_symid = symbol_add("endlon");
  _startlat_symid = symbol_add("startlat");
  _endlat_symid = symbol_add("endlat");
  _spacing_symid = symbol_add("spacing");
  _resolution_symid = symbol_add("resolution");
}

void GraticuleFCFunc::execute() {
  ComValue startlon(stack_key(_startlon_symid));
  ComValue endlon(stack_key(_endlon_symid));
  ComValue startlat(stack_key(_startlat_symid));
  ComValue endlat(stack_key(_endlat_symid));
  ComValue spacing(stack_key(_spacing_symid));
  ComValue resolution(stack_key(_resolution_symid));
  ComValue result;

  float slon = -180.0;
  float elon = 180.0;
  float slat = -90.0;
  float elat = 90.0;
  float spc = 15.0;
  float res = 5.0;

  if (startlon.is_num())
    slon = startlon.float_val();
  if (endlon.is_num())
    elon = endlon.float_val();
  if (startlat.is_num())
    slat = startlat.float_val();
  if (endlat.is_num())
    elat = endlat.float_val();
  if (spacing.is_num())
    spc = spacing.float_val();
  if (resolution.is_num())
    res = resolution.float_val();

  float lonspan = elon - slon;
  int numlon = int(lonspan / spc) + 1;
  float latspan = elat - slat;
  int numlat = int(latspan / spc) + 1;

  MapFeature** feats = new MapFeature*[numlon + numlat];
  int fi = 0;
  for (float i = slon; i <= elon; i += spc) {
    int npts = int(latspan / res) + 1;
    float* x = new float[npts];
    float* y = new float[npts];
    int ip = 0;
    for (float j = slat; j <= elat; j += res) {
      x[ip] = i;
      y[ip++] = j;
    }
    MapFeature* f = new MapFeature(-1, MapFeature::LineType);
    MapEdge* line = new MapEdge(f);
    line->insert_pointers(npts, x, y);
    f->edge_primitive(line);
    feats[fi++] = f;
  }
  for (float i = slat; i <= elat; i += spc) {
    int npts = int(lonspan / res) + 1;
    float* x = new float[npts];
    float* y = new float[npts];
    int ip = 0;
    for (float j = slon; j <= elon; j += res) {
      x[ip] = j;
      y[ip++] = i;
    }
    MapFeature* f = new MapFeature(-1, MapFeature::LineType);
    MapEdge* line = new MapEdge(f);
    line->insert_pointers(npts, x, y);
    f->edge_primitive(line);
    feats[fi++] = f;
  }
  MapFeatureClass* fc = new MapFeatureClass(fi, feats, "graticule");
  MapFClassComp* fccomp = new MapFClassComp(fc);
  for (int i = 0; i < fi; i++) {
    feats[i]->feature_class(fc);
    MapFeatureComp* fcomp = new MapFeatureComp(nil, fccomp);
    fcomp->feature(feats[i]);
    fccomp->Append(fcomp);
  }
  ComponentView* view = new ComponentView(fccomp);
  result.type(AttributeValue::ObjectType);
  result.obj_ref() = view;
  result.obj_type_ref() = FEATURE_CLASS;
  push_stack(result);
}
